/*
	Marco Signorini - www.EtherMania.com 
	High Power LED shield sketch example
	
	This sketch generates a fixed sequence of colors. 
	Each color smootly changes to next color in the sequence
	It could be used as main example for testing the 
	4 channel High Power LED driver shield made by EtherMania.
*/

#define RED_PWMPIN    3
#define GREEN_PWMPIN  9
#define BLUE_PWMPIN  10
#define WHITE_PWMPIN 11

#define TRANSITION_STEPS 80

#define NUMSTEPS  16

unsigned char redSteps[NUMSTEPS] =   { 0, 50, 255, 255, 100, 150, 200, 230, 0, 60, 20, 30, 30, 0, 150, 255 };
unsigned char greenSteps[NUMSTEPS] = { 100, 100, 0, 50, 200, 250, 255, 100, 255, 100, 150, 80, 0, 100, 0, 0 };
unsigned char blueSteps[NUMSTEPS] =  { 0, 255, 255, 0,  0, 0, 100, 150, 0, 100, 250, 255, 255, 180, 90, 0 };
unsigned char whiteSteps[NUMSTEPS] = { 0, 0, 50, 50, 100, 100, 150, 150, 200, 200, 150, 100, 50, 0, 0, 0 };

void setup()  { 
} 

void loop() {
  for (unsigned char step_i = 0; step_i < NUMSTEPS; step_i++) 
  {
    morphToStep(step_i);
    delay(2000);    
  }
}

void morphToStep(unsigned char newStep) {

  unsigned char prevStep = (newStep - 1) & 0x0F;
  
  unsigned char curRed = redSteps[prevStep];
  unsigned char curGreen = greenSteps[prevStep];
  unsigned char curBlue = blueSteps[prevStep];
  unsigned char curWhite = whiteSteps[prevStep];
  
  unsigned char nextRed = redSteps[newStep];
  unsigned char nextGreen = greenSteps[newStep];
  unsigned char nextBlue = blueSteps[newStep];
  unsigned char nextWhite = whiteSteps[newStep];
  
  while ((curRed != nextRed) || (curGreen != nextGreen) ||
         (curBlue != nextBlue) || (curWhite != nextWhite))
  {
      curRed = updateBrightness(curRed, nextRed);
      curGreen = updateBrightness(curGreen, nextGreen);
      curBlue = updateBrightness(curBlue, nextBlue);
      curWhite = updateBrightness(curWhite, nextWhite);
  
      updatePWM(curRed, curGreen, curBlue, curWhite);
      delay(20);
  }
}

unsigned char updateBrightness(unsigned char current, unsigned char target) {
  
  if (current == target)
    return current;
    
  if (current > target)
    current = current - 1;
  else
    current = current + 1;
    
  return current;
}

void updatePWM(unsigned char red, unsigned char green, unsigned char blue, unsigned char white) {
  
  analogWrite(RED_PWMPIN, red);
  analogWrite(GREEN_PWMPIN, green);
  analogWrite(BLUE_PWMPIN, blue);
  analogWrite(WHITE_PWMPIN, white);
}
